// Copyright (C) 2004 MySQL AB
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 2 as published by
// the Free Software Foundation
//
// There are special exceptions to the terms and conditions of the GPL 
// as it is applied to this software. View the full text of the 
// exception in file EXCEPTIONS in the directory of this software 
// distribution.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 

using System;
using MySql.Data.Common;

namespace MySql.Data.MySqlClient
{
	/// <summary>
	/// Summary description for Field41.
	/// </summary>
	internal class MySqlField41 : MySqlField
	{

		public MySqlField41(Driver driver)
		{
			this.encoding = driver.Encoding;
			Packet packet = driver.ReadPacket();
			bool is411 = driver.Version.isAtLeast(4,1,1);

			if (is411)
				CatalogName = packet.ReadLenString();
			DatabaseName = packet.ReadLenString();
			TableName = packet.ReadLenString();
			RealTableName = packet.ReadLenString();
			ColumnName = packet.ReadLenString();
			string origName = packet.ReadLenString();
			packet.ReadByte();
			CharactetSetIndex = packet.ReadInteger(2);
			if (is411)
				ColumnLength = packet.ReadInteger(4);
			else
				ColumnLength = packet.ReadInteger(3);
			MySqlDbType type = (MySqlDbType)packet.ReadByte();

			if ((driver.Flags & ClientFlags.LONG_FLAG) != 0) 
				Flags = (ColumnFlags)packet.ReadInteger(2);
			else 
				Flags = (ColumnFlags)packet.ReadByte();

			byte scale = (byte)packet.ReadByte();

			InitValue( type, scale );
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="packet"></param>
		/// <param name="version"></param>
		/// <returns></returns>
		public int SetBinaryData( Packet packet, DBVersion version )
		{
			bufIndex = 0;

			if (packet == null) 
			{
				colValue.IsNull = true;
				buffer = null;
				return 0;
			}

			buffer = packet.GetBuffer();
			bufIndex = packet.Position;
			colValue.SetBinaryData( packet );
			bufLength = packet.Position - bufIndex;
			return (int)bufLength;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="buf"></param>
		/// <param name="index"></param>
		/// <param name="len"></param>
		/// <param name="version"></param>
		public override void SetTextData( byte[] buf, long index, long len, DBVersion version )
		{
			base.SetTextData( buf, index, len, version );
		}

	}

}
