using System;
using System.IO;
using System.Text;

namespace MySql.Data.MySqlClient
{
	/// <summary>
	/// Summary description for PacketWriter.
	/// </summary>
	internal class PacketWriter
	{
		private Stream		stream;
		private Encoding	encoding;

		public PacketWriter(Encoding encoding) 
		{
			stream = new MemoryStream();
			this.encoding = encoding;
		}

		public PacketWriter(Stream stream, Encoding encoding)
		{
			this.stream = stream;
			this.encoding = encoding;
		}

		public Stream Stream 
		{
			get { return stream; }
		}

		public void WriteByte( byte b ) 
		{
			stream.WriteByte( b );
		}

		public void Write( byte[] buffer ) 
		{
			Write( buffer, 0, buffer.Length );
		}

		public void Write( byte[] buffer, int offset, int count ) 
		{
			stream.Write( buffer, offset, count );
		}

		public void WriteLength( long length ) 
		{
			if (length < 251)
				WriteByte( (byte)length );
			else if ( length < 65536L )
			{
				WriteByte( 252 );
				WriteInteger( length, 2 );
			}
			else if ( length < 16777216L )
			{
				WriteByte( 253 );
				WriteInteger( length, 3 );
			}
			else 
			{
				WriteByte( 254 );
				WriteInteger( length, 4 );
			}
		}

		/// <summary>
		/// WriteInteger
		/// </summary>
		/// <param name="v"></param>
		/// <param name="numbytes"></param>
		public void WriteInteger( long v, int numbytes )
		{
			long val = v;

			if (numbytes < 1 || numbytes > 4) 
				throw new ArgumentOutOfRangeException("Wrong byte count for WriteInteger");

			for (int x=0; x < numbytes; x++)
			{
				stream.WriteByte( (byte)(val&0xff) );
				val >>= 8;
			}
		}

		public void WriteLenString( string s )
		{
			WriteLength( s.Length );
			WriteStringNoNull( s );
		}

		public void WriteString(string v )
		{
			WriteStringNoNull( v );
			stream.WriteByte(0);
		}
 
		public void WriteStringNoNull(string v )
		{
			byte[] bytes = encoding.GetBytes(v);
			stream.Write(bytes, 0, bytes.Length);
		}

	}
}
