// Copyright (C) 2004 MySQL AB
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 2 as published by
// the Free Software Foundation
//
// There are special exceptions to the terms and conditions of the GPL 
// as it is applied to this software. View the full text of the 
// exception in file EXCEPTIONS in the directory of this software 
// distribution.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 

using System;
using System.Collections;

namespace MySql.Data.MySqlClient
{
	/// <summary>
	/// Summary description for PreparedStatement.
	/// </summary>
	internal class PreparedStatement
	{
		private InternalConnection	conn;
		private int					statementId;
		private int					numColumns;
		private int					numParameters;
		private MySqlField41[]		metaData;
		private int					executionCount;

		public PreparedStatement(InternalConnection conn, int statementId, int numC, int numP)
		{
			this.conn = conn;
			this.statementId = statementId;
			numColumns = numC;
			numParameters = numP;
			executionCount = 0;
		}

		#region Properties

		public int StatementId 
		{
			get { return statementId; }
		}

		public int NumColumns 
		{
			get { return numColumns; }
		}

		public int NumParameters 
		{
			get { return numParameters; }
		}

		public int ExecutionCount 
		{
			get { return executionCount; }
		}

		#endregion

		public CommandResult Execute( MySqlParameterCollection parameters )
		{
			PacketWriter packet = new PacketWriter(System.Text.Encoding.GetEncoding("utf-8"));
			//Packet packet = conn.Driver.CreatePacket(null);

			//TODO: support long data here
			// create our null bitmap
			BitArray nullMap = new BitArray( metaData.Length );
			for (int x=0; x < metaData.Length; x++)
			{
				if (parameters[x].Value == DBNull.Value)
					nullMap[x] = true;
			}
			byte[] nullMapBytes = new byte[ (metaData.Length + 7)/8 ];
			nullMap.CopyTo( nullMapBytes, 0 );

			// start constructing our packet
			packet.WriteInteger( StatementId, 4 );
			packet.WriteByte( 0 );          // flags; always 0 for 4.1
			packet.WriteInteger( 1, 4 );    // interation count; 1 for 4.1
			packet.Write( nullMapBytes );
			if (metaData != null && metaData.Length > 0)
				packet.WriteByte( 1 );			// rebound flag
			else
				packet.WriteByte( 0 );
			//TODO:  only send rebound if parms change

			// write out the parameter types
			foreach (MySqlField41 field in metaData)
			{
				MySqlParameter parm = parameters[ field.ColumnName ];
				packet.WriteInteger( (long)parm.MySqlDbType, 2 ); 
			}
				//field.MySqlValue.MySqlDbType, 2 );
//			foreach (MySqlParameter parm in parameters)
//				packet.WriteInteger( (int)parm.MySqlDbType, 2 );

			// now write out all non-null values
			foreach (MySqlField41 field in metaData)
			//foreach (MySqlParameter parm in parameters)
			{
				MySqlParameter parm = parameters[ field.ColumnName ];
				if (parm.Value == DBNull.Value) continue;
				parm.SerializeToBinary( packet );
			}

			executionCount ++;
			// send the data packet and return the CommandResult
			return conn.ExecuteStatement( ((System.IO.MemoryStream)packet.Stream).ToArray() );
		}

		public void LoadMetaData(string[] nameOrder)
		{
			metaData = new MySqlField41[ numParameters ];

			for (int x=0; x < metaData.Length; x++)
			{
				metaData[x] = new MySqlField41( conn.Driver );
				metaData[x].ColumnName = nameOrder[x];
			}

			// Use CommandResult class to read off the fields
			CommandResult cr = new CommandResult(conn.Driver);
			for (int x=0; x<numColumns; x++)
				cr.GetField();

			// read off last packet
			Packet last = conn.Driver.ReadPacket();
		}

	}
}
