// Copyright (C) 2004 MySQL AB
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 2 as published by
// the Free Software Foundation
//
// There are special exceptions to the terms and conditions of the GPL 
// as it is applied to this software. View the full text of the 
// exception in file EXCEPTIONS in the directory of this software 
// distribution.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 

using System;
using System.Data;
using MySql.Data.MySqlClient;

namespace MySql.Data.Types
{
	/// <summary>
	/// Summary description for MySqlBinary
	/// </summary>
	internal class MySqlBinary : MySqlValue
	{
		private object	mValue;
		private bool	isBinary;
		private ulong	length;
		private ulong	index;

		public MySqlBinary() 
		{
			dbType = DbType.Binary;
			mySqlDbType = MySqlDbType.Blob;
		}

		public MySqlBinary(MySqlDbType type) : this()
		{
			mySqlDbType = type;
		}

		public MySqlBinary(byte[] val, MySqlDbType type) : this(type)
		{
			mValue = val;
		}

		internal string ToString( System.Text.Encoding encoding ) 
		{
			if (mValue is string) return (string)mValue;
			if (mValue is byte[])
				return encoding.GetString( (byte[])mValue, (int)index, (int)length );
			return mValue.ToString();
		}

		internal override void SerializeBinary(PacketWriter writer)
		{
			if (mValue is byte[])
			{
				byte[] ar = (mValue as byte[]);
				writer.WriteLength( ar.Length );
				writer.Write( ar, 0, ar.Length );
			}
		}

		public override byte[] ToBytes()
		{
			return (byte[])mValue;
		}

		public bool IsBinary 
		{
			get { return isBinary; }
			set { isBinary = value; }
		}

		public ulong Index 
		{
			get { return index; }
			set { index = value; }
		}

		public ulong Length 
		{
			get { return length; }
			set { length = value; }
		}

		internal override object ValueAsObject
		{
			get { return mValue; }
			set { Value = value; }
		}


		public object Value
		{
			get { return mValue; }
			set 
			{ 
				if (value is byte[] || value is Array)
					isBinary = true;
				else
					isBinary = false;
				mValue = value; 
			} 
		}

		public override void Parse(string s)
		{
			mValue = System.Text.Encoding.Default.GetBytes(s);
		}

		internal override DbType DbType
		{
			get
			{
				if (isBinary) return DbType.Binary;
				return DbType.String;
			}
		}


		internal override Type SystemType
		{
			get 
			{
				if (isBinary)
					return typeof(byte[]);
				return typeof(string);
			}
		}

		internal override string GetMySqlTypeName()
		{
			switch (mySqlDbType) 
			{
				case MySqlDbType.TinyBlob:		return "TINY_BLOB";
				case MySqlDbType.MediumBlob:	return "MEDIUM_BLOB";
				case MySqlDbType.LongBlob:		return "LONG_BLOB";
				case MySqlDbType.Blob:			
				default:
					return "BLOB";
			}
		}

		internal override long SetBinaryData( Packet packet )
		{
			mValue = packet.GetBuffer();
			length = (ulong)packet.ReadPackedInteger();
			index = (ulong)packet.Position;
			packet.Position += (long)length;
			return 0;
		}

	}
}
