// Copyright (C) 2004 MySQL AB
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 2 as published by
// the Free Software Foundation
//
// There are special exceptions to the terms and conditions of the GPL 
// as it is applied to this software. View the full text of the 
// exception in file EXCEPTIONS in the directory of this software 
// distribution.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 

using System;
using System.Data;
using MySql.Data.MySqlClient;

namespace MySql.Data.Types
{
	/// <summary>
	/// Summary description for MySqlDecimal.
	/// </summary>
	internal class MySqlDecimal : MySqlNumber
	{
		private byte	precision;
		private byte	scale;
		private Decimal	mValue;

		public MySqlDecimal() : base()
		{
			dbType = DbType.Decimal;
			mySqlDbType = MySqlDbType.Decimal;
		}

		public MySqlDecimal(Decimal val) : this()
		{
			mValue = val;
		}

		public byte Precision 
		{
			get { return precision; }
			set { precision = value; }
		}

		public byte Scale 
		{
			get { return scale; }
			set { scale = value; }
		}

		internal override void SerializeBinary(PacketWriter writer)
		{
			//TODO: fix this
		}


		public override byte[] ToBytes()
		{
			throw new NotSupportedException("Converting decimal to bytes isn't currently supported");
		}

		internal override object ValueAsObject
		{
			get { return mValue; }
			set { mValue = Convert.ToDecimal(value); }
		}


		public Decimal Value
		{
			get { return mValue; }
			set { mValue = value; }
		}

		public override void Parse(string s)
		{
			mValue = Decimal.Parse( s, numberFormat );
		}

		internal override Type SystemType
		{
			get { return typeof(Decimal); }
		}

		internal override string GetMySqlTypeName()
		{
			return "DECIMAL";
		}

		internal override long SetBinaryData(MySql.Data.MySqlClient.Packet packet)
		{
			string sValue = packet.ReadLenString();
			Parse( sValue );
			return sValue.Length;
		}

	}
}
