// Copyright (C) 2004 MySQL AB
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 2 as published by
// the Free Software Foundation
//
// There are special exceptions to the terms and conditions of the GPL 
// as it is applied to this software. View the full text of the 
// exception in file EXCEPTIONS in the directory of this software 
// distribution.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 

using System;
using System.Data;
using MySql.Data.MySqlClient;

namespace MySql.Data.Types
{
	/// <summary>
	/// Summary description for MySqlInt32.
	/// </summary>
	internal class MySqlInt32 : MySqlNumber
	{
		private int	mValue;

		public MySqlInt32() : base()
		{
			dbType = DbType.Int32;
			mySqlDbType = MySqlDbType.Int32;
		}

		public MySqlInt32(MySqlDbType type) : this()
		{
			mySqlDbType = type;
		}

		internal override void SerializeBinary(PacketWriter writer)
		{
			writer.Write( BitConverter.GetBytes(mValue) );
		}

		public override byte[] ToBytes()
		{
			return BitConverter.GetBytes(mValue);
		}

		internal override object ValueAsObject 
		{
			get { return mValue; }
			set { mValue = Convert.ToInt32(value); }
		}

		public int Value
		{
			get { return mValue; }
			set { mValue = value; }
		}

		public override void Parse(string s)
		{
			mValue = Int32.Parse( s );
		}

		internal override Type SystemType
		{
			get { return typeof(Int32); }
		}

		internal override string GetMySqlTypeName()
		{
			if (mySqlDbType == MySqlDbType.Int24)
				return "MEDIUMINT";
			return "INT";
		}

		internal override long SetBinaryData( Packet packet )
		{
			int cnt = 4;
			if (mySqlDbType == MySqlDbType.Int24)
				cnt = 3;
			mValue = packet.ReadInteger(cnt);
			return cnt;
		}

	}
}
