// Copyright (C) 2004 MySQL AB
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 2 as published by
// the Free Software Foundation
//
// There are special exceptions to the terms and conditions of the GPL 
// as it is applied to this software. View the full text of the 
// exception in file EXCEPTIONS in the directory of this software 
// distribution.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 

using System;
using System.Data;
using MySql.Data.MySqlClient;

namespace MySql.Data.Types
{
	/// <summary>
	/// Summary description for MySqlString.
	/// </summary>
	internal class MySqlString : MySqlValue
	{
		private string mValue;

		public MySqlString() 
		{
			dbType = DbType.String;
			mySqlDbType = MySqlDbType.VarChar;
		}

		public MySqlString(MySqlDbType type) : this()
		{
			dbType = DbType.String;
			mySqlDbType = type;
		}

		public MySqlString(string val, MySqlDbType type) : this( type )
		{
			mValue = val;
		}

		internal override void SerializeBinary(PacketWriter writer)
		{
			writer.WriteLenString( mValue );
		}

		public byte[] ToBytes( System.Text.Encoding encoding ) 
		{
			Packet p = new Packet(false);
			p.Encoding = encoding;
			p.WriteLenString( mValue );
			byte[] buff = new byte[ p.Length ];
			Array.Copy( p.GetBuffer(), 0, buff, 0, p.Length );
			return buff;
		}

		public override byte[] ToBytes()
		{
			return System.Text.Encoding.Default.GetBytes(mValue);
		}

		internal override object ValueAsObject
		{
			get { return mValue; }
			set { mValue = value.ToString(); }
		}

		public string Value
		{
			get { return mValue; }
			set { mValue = value; }
		}

		public override void Parse(string s)
		{
			mValue = s;
		}

		internal override Type SystemType
		{
			get { return typeof(String); }
		}

		internal override string GetMySqlTypeName()
		{
			switch (mySqlDbType) 
			{
				case MySqlDbType.Set: return "SET";
				case MySqlDbType.Enum: return "ENUM";
			}
			return "VARCHAR";
		}

		internal override long SetBinaryData(Packet packet)
		{
			long start = packet.Position;
			packet.ReadPackedInteger();
			long pos = packet.Position;

			packet.Position = start;
			mValue = packet.ReadLenString();
			return packet.Position - pos;
		}

	}
}
