// Copyright (C) 2004 MySQL AB
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 2 as published by
// the Free Software Foundation
//
// There are special exceptions to the terms and conditions of the GPL 
// as it is applied to this software. View the full text of the 
// exception in file EXCEPTIONS in the directory of this software 
// distribution.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 

using System;
using System.Data;
using MySql.Data.MySqlClient;

namespace MySql.Data.Types
{
	/// <summary>
	/// Summary description for MySqlDateTime.
	/// </summary>
	internal class MySqlTimeSpan : MySqlValue
	{
		private TimeSpan	mValue;

		public MySqlTimeSpan() 
		{
			dbType = DbType.Time;
			mySqlDbType = MySqlDbType.Time;
		}

		public MySqlTimeSpan(MySqlDbType type) : this()
		{
			mySqlDbType = type;
		}

		public MySqlTimeSpan(TimeSpan val, MySqlDbType type) : this(type)
		{
			mValue = val;
		}

		internal override void SerializeBinary(PacketWriter writer)
		{
			writer.WriteByte( 8 );
			writer.WriteByte( (byte)(mValue.TotalSeconds < 0 ? 1 : 0 ));
			writer.WriteInteger( mValue.Days, 4 );
			writer.WriteByte( (byte)mValue.Hours );
			writer.WriteByte( (byte)mValue.Minutes );
			writer.WriteByte( (byte)mValue.Seconds );
		}

		public override byte[] ToBytes()
		{
			Packet p = new Packet(false);

			p.WriteByte( 8 );
			p.WriteByte( (byte)(mValue.TotalSeconds < 0 ? 1 : 0 ));
			p.WriteInteger( mValue.Days, 4 );
			p.WriteByte( (byte)mValue.Hours );
			p.WriteByte( (byte)mValue.Minutes );
			p.WriteByte( (byte)mValue.Seconds );
			//p.WriteInteger( mValue.Minutes, 4 );

			byte[] buff = new byte[ p.Length ];
			Array.Copy( p.GetBuffer(), 0, buff, 0, p.Length );
			return buff;
		}

		public override string ToString()
		{
			return String.Format("{0} {1:00}:{2:00}:{3:00}.{4}", 
				mValue.Days, mValue.Hours, mValue.Minutes, mValue.Seconds, mValue.Milliseconds );
		}


		internal override string GetMySqlTypeName()
		{
			return "TIME";
		}

		internal override object ValueAsObject
		{
			get { return mValue; }
			set { mValue = (TimeSpan)value; }
		}

		public TimeSpan Value
		{
			get { return mValue; }
			set { mValue = value; } 
		}

		public override void Parse(string s)
		{
			mValue = TimeSpan.Parse(s);
		}

		internal void ParseMySql( string s, bool is41 ) 
		{
			string[] parts = s.Split(':');
			int hours = Int32.Parse( parts[0] );
			int mins = Int32.Parse( parts[1] );
			int secs = Int32.Parse( parts[2] );
			int days = hours / 24;
			hours = hours - (days * 24);
			mValue = new TimeSpan( days, hours, mins, secs, 0 );
		}

		internal override Type SystemType
		{
			get { return typeof(TimeSpan); }
		}

		internal override long SetBinaryData(Packet packet)
		{
			long bufLength = 0;

			bufLength = packet.ReadByte();
			int negate = 0;
			if (bufLength > 0)
				negate = packet.ReadByte();

			if (bufLength == 0)
				IsNull = true;
			else if (bufLength == 5)
				mValue = new TimeSpan( packet.ReadInteger(4), 0, 0, 0 );
			else if (bufLength == 8)
				mValue = new TimeSpan( packet.ReadInteger(4), 
					packet.ReadByte(), packet.ReadByte(), packet.ReadByte() );
			else 
				mValue = new TimeSpan( packet.ReadInteger(4), 
					packet.ReadByte(), packet.ReadByte(), packet.ReadByte(),
					packet.ReadInteger(4) / 1000000 );

			if (negate == 1)
				mValue = mValue.Negate();
			return bufLength;
		}

		public string ToMySqlString(bool is41) 
		{
			return mValue.ToString();
		}
	}
}
