// Copyright (C) 2004 MySQL AB
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 2 as published by
// the Free Software Foundation
//
// There are special exceptions to the terms and conditions of the GPL 
// as it is applied to this software. View the full text of the 
// exception in file EXCEPTIONS in the directory of this software 
// distribution.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 

using System;
using System.Data;
using MySql.Data.MySqlClient;

namespace MySql.Data.Types
{
	/// <summary>
	/// Summary description for MySqlInt16.
	/// </summary>
	internal class MySqlUInt16 : MySqlNumber
	{
		private ushort	mValue;

		public MySqlUInt16() : base()
		{
			dbType = DbType.UInt16;
			mySqlDbType = MySqlDbType.Int16;
		}

		public MySqlUInt16(MySqlDbType type) : this()
		{
			mySqlDbType = type;
		}

		internal override void SerializeBinary(PacketWriter writer)
		{
			writer.Write( BitConverter.GetBytes( mValue ) );
		}

		public override byte[] ToBytes()
		{
			return BitConverter.GetBytes( mValue );
		}

		internal override object ValueAsObject
		{
			get { return mValue; }
			set { mValue = Convert.ToUInt16(value); }
		}

		public ushort Value
		{
			get { return mValue; }
			set { mValue = value; }
		}

		public override void Parse(string s)
		{
			mValue = UInt16.Parse( s );
		}

		internal override Type SystemType
		{
			get { return typeof(UInt16); }
		}

		internal override string GetMySqlTypeName()
		{
			return "SMALLINT";
		}

		internal override long SetBinaryData( Packet packet )
		{
			mValue = BitConverter.ToUInt16( packet.GetBuffer(), (int)packet.Position );
			return 2;
		}

	}
}
