// Copyright (C) 2004 MySQL AB
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 2 as published by
// the Free Software Foundation
//
// There are special exceptions to the terms and conditions of the GPL 
// as it is applied to this software. View the full text of the 
// exception in file EXCEPTIONS in the directory of this software 
// distribution.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 

using System;
using NUnit.Framework;
using System.Configuration;
using System.Data;
using MySql.Data.MySqlClient;

namespace MySql.Data.MySqlClient.Tests
{
	/// <summary>
	/// Summary description for BaseTest.
	/// </summary>
	public class BaseTest
	{
		protected MySqlConnection	conn;
		protected string			table;
		protected string			csAdditions = String.Empty;

		protected void Open()
		{
			try 
			{
				string connString = ConfigurationSettings.AppSettings["ConnectionString"];
				connString += csAdditions;
				conn = new MySqlConnection( connString );
				conn.Open();
			}
			catch (Exception ex)
			{
				System.Diagnostics.Trace.WriteLine(ex.Message);
			}
		}

		protected void Close()
		{
			// delete the table we created.
			if (conn.State == ConnectionState.Closed)
				conn.Open();
			execSQL("DROP TABLE IF EXISTS Test");
			conn.Close();
		}

		protected bool Is50 
		{ 
			get { return conn.ServerVersion.StartsWith("5.0"); }
		}

		protected bool Is41 
		{
			get { return conn.ServerVersion.StartsWith("4.1"); }
		}

		protected bool Is40
		{
			get { return conn.ServerVersion.StartsWith("4.0"); }
		}

		[SetUp]
		protected virtual void Setup() 
		{
			IDataReader reader = execReader("SHOW TABLES LIKE 'test'");
			bool exists = reader.Read();
			reader.Close();
			if (exists)
				execSQL( "TRUNCATE TABLE test" );
			if (Is50) 
			{
				execSQL("DROP PROCEDURE IF EXISTS spTest");
				execSQL("DROP FUNCTION IF EXISTS fnTest");
			}
		}

		[TearDown]
		protected void Teardown()
		{
		}

		protected void execSQL( string sql )
		{
			MySqlCommand cmd = new MySqlCommand(sql, conn);
			cmd.ExecuteNonQuery();
		}

		protected IDataReader execReader( string sql )
		{
			MySqlCommand cmd = new MySqlCommand(sql, conn);
			return cmd.ExecuteReader();
		}
	}
}
