// Copyright (C) 2004 MySQL AB
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 2 as published by
// the Free Software Foundation
//
// There are special exceptions to the terms and conditions of the GPL 
// as it is applied to this software. View the full text of the 
// exception in file EXCEPTIONS in the directory of this software 
// distribution.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 

using System;
using NUnit.Framework;
using MySql.Data.MySqlClient;

namespace MySql.Data.MySqlClient.Tests
{
	/// <summary>
	/// Summary description for CharacterSets.
	/// </summary>
	public class CharacterSets : BaseTest
	{
		[TestFixtureSetUp]
		public void FixtureSetup()
		{
			Open();
		}

		[TestFixtureTearDown]
		public void FixtureTeardown()
		{
			Close();
		}

		[Test]
		public void Unicode()
		{
			MySqlCommand cmd = new MySqlCommand(
				"DROP TABLE IF EXISTS test; " +
				"CREATE TABLE test (u1 varchar(255) CHARACTER SET utf8, u2 varchar(255) CHARACTER SET ucs2)", conn);
			cmd.ExecuteNonQuery();

			String teststring = "\u0120\u0121\u0125\u0411";

			cmd.CommandText = "INSERT INTO test VALUES(?p1, ?p2)";
			cmd.Parameters.Add( "?p1", teststring );
			cmd.Parameters.Add( "?p2", teststring );
			cmd.ExecuteNonQuery();

			cmd.CommandText = "SELECT * FROM test";
			MySqlDataReader reader = cmd.ExecuteReader();
			reader.Read();
			string s1 = reader.GetString(0);
			string s2 = reader.GetString(1);
			reader.Close();

			Assert.AreEqual( teststring, s1 );
			Assert.AreEqual( teststring, s2 );
		}
	}
}
