// Copyright (C) 2004 MySQL AB
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 2 as published by
// the Free Software Foundation
//
// There are special exceptions to the terms and conditions of the GPL 
// as it is applied to this software. View the full text of the 
// exception in file EXCEPTIONS in the directory of this software 
// distribution.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 

using System;
using System.Data;
using System.IO;
using NUnit.Framework;

namespace MySql.Data.MySqlClient.Tests
{
	[TestFixture()]
	public class Syntax : BaseTest
	{
		[TestFixtureSetUp]
		public void FixtureSetup()
		{
			Open();

			execSQL("DROP TABLE IF EXISTS Test");
			execSQL("CREATE TABLE Test (id INT NOT NULL, name VARCHAR(100), PRIMARY KEY(id))");
		}

		[TestFixtureTearDown]
		public void FixtureTeardown()
		{
			Close();
		}

		[Test()]
		public void ShowCreateTable()
		{
			MySqlDataAdapter da = new MySqlDataAdapter("SHOW CREATE TABLE test", conn);
			DataTable dt = new DataTable();
			da.Fill(dt);

			Assert.AreEqual( 1, dt.Rows.Count );
			Assert.AreEqual( 2, dt.Columns.Count );
		}

		[Test()]
		public void CRInSQL()
		{
			string sql = "INSERT INTO test (id, name) VALUES(1, 'Test');\r\nSELECT * FROM test";
			MySqlCommand cmd = new MySqlCommand(sql, conn);
			MySqlDataReader reader = cmd.ExecuteReader();
			bool result = reader.Read();
			Assert.AreEqual( true, result );
			Assert.AreEqual( 1, reader.GetInt32(0));
			Assert.AreEqual( "Test", reader.GetString(1));
			reader.Close();
		}

		[Test()]
		public void LoadDataLocalInfile() 
		{
			execSQL("set @@global.max_allowed_packet=4000000");

			MySqlConnection c = new MySqlConnection( conn.ConnectionString + ";pooling=false" );
			c.Open();

			string path = Path.GetTempFileName();
			StreamWriter sw = new StreamWriter( path );
			for (int i=0; i < 200000; i++) 
				sw.WriteLine(i + ",'Test'");
			sw.Flush();
			sw.Close();

			path = path.Replace(@"\", @"\\");
			MySqlCommand cmd = new MySqlCommand("LOAD DATA LOCAL INFILE '" + path + "' INTO TABLE Test", c);
			object cnt = cmd.ExecuteNonQuery();
			Assert.AreEqual( 200000, cnt );

			cmd.CommandText = "SELECT COUNT(*) FROM Test";
			cnt = cmd.ExecuteScalar();
			Assert.AreEqual( 200000, cnt );

			c.Close();
			execSQL("set @@global.max_allowed_packet=1047256");
		}
	}
}
